/*
   This file is part of msplot, a visualisation tool for Massif output

   Copyright (C) 2007-2008 Martijn Versteegh


   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.

   The GNU General Public License is contained in the file COPYING.
*/



// this file implements a RadialPlot widget for wxWidgets, it doesn't depend on any other part of msplot, so you can easily re-use it
// for other projects.

// wxWidgets drawing code sucks though, so it doesn't really look to good.

#ifndef __RADIALPLOT_H__
#define __RADIALPLOT_H__

#include "wx/window.h"
#include "wx/bitmap.h"
#include "wx/dcclient.h"
#include "wx/dcmemory.h"
#include <math.h>

class RadialPlotTreeItem
{
	public:
		unsigned int level;
		double startAngle;
		double endAngle;
		wxColor color;
		unsigned int id;
		void const * handle;
};



class RadialPlot
	: public wxWindow
{
	public:
		RadialPlot(wxWindow *parent)
			: wxWindow(parent,-1)
		{
			maxNumItems = 32;
			numItems = 0;
			items = new RadialPlotTreeItem[maxNumItems];
			backBuffer = NULL;
			maxLevel = 0;
			maxValue = 0;
			autoLevel = false;
			selected = (unsigned int)(-1);
		}
		
		virtual ~RadialPlot()
		{
			delete [] items;
			delete backBuffer;
		}
		
		
		void RefreshTree(void const *startNode, int _maxLevel, int _maxValue);

		wxColor GetItemColor(void const *handle)
		{
			unsigned int item = FindItem(handle);

			if (item == (unsigned int)(-1))
				return wxColor(0,0,0);

			return items[item].color;
		}

		wxColor GetItemColor(unsigned int id)
		{
			unsigned int item = FindItem(id);

			if (item == (unsigned int)(-1))
				return wxColor(0,0,0);

			return items[item].color;
		}

		wxPoint GetItemCentre(void const *handle);
		wxPoint GetItemCentre(unsigned int id);

		virtual void OnMouseEvt(wxMouseEvent &event, void const *handle)	// pass mouse events with prefilled item handle
		{
		
		}
		// these get called with e DC associated with the backbuffer bitmap (the control is
		// drawn double-buffered for easy refreshing/anti-flickering)
		// before and after the graph is drawn
		// you can use this to add your own stuff under or over the graph
		virtual void  CustomDrawPre(wxDC *bitmap)
		{
		}
		
		virtual void  CustomDrawPost(wxDC *bitmap)
		{

		}

		virtual unsigned int GenerateColor(unsigned int c);
		
	private:
		DECLARE_EVENT_TABLE();

		void OnMouse(wxMouseEvent &evt);

		virtual void const *GetRootNode() = 0;
		virtual unsigned GetNumChildren(void const *handle) = 0;
		virtual void const *GetChild(void const *handle, int child) = 0;
		virtual void const *GetParent(void const *handle) = 0;
		virtual unsigned int GetValue(void const *handle) = 0;
		virtual unsigned int GetId(void const *handle) = 0;
		virtual char const * GetName(void const *handle) = 0;
		unsigned int maxNumItems, numItems, selected;
		unsigned int maxLevel, maxValue;
		RadialPlotTreeItem *items;

		bool autoLevel;

		void ScanTree(void const *node, unsigned int level, double startAngle, double endAngle);

		void Grow()
		{
			maxNumItems *= 2;
			RadialPlotTreeItem *tmp = new RadialPlotTreeItem[maxNumItems];

			for (unsigned int i = 0; i < numItems; i++)
			{
				tmp[i] = items[i];
			}

			delete [] items;
			items = tmp;
		}

		void OnPaint(wxPaintEvent &evt);
		void OnSize(wxSizeEvent &evt);
		void OnErase(wxEraseEvent &evt);

		void Draw(wxDC *dc = NULL);


		unsigned int HitTest(int x, int y);
		unsigned int FindItem(void const *handle);
		unsigned int FindItem(unsigned int id);

		wxPoint GetItemCentreWithIndex(unsigned int index);

		void DrawItems(wxDC *onto);


		void DrawItem(wxDC * onto, RadialPlotTreeItem const &item, wxPoint c, int diameter);

		wxBitmap *backBuffer;
	
};


#endif
